package com.flyfish.framework.file.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.MediaType;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.function.Function;

@AllArgsConstructor
@Getter
public enum MediaTypes {

    OCT_STREAM(Collections.emptyList(), extend -> MediaType.APPLICATION_OCTET_STREAM),
    IMAGE(Arrays.asList("jpg", "jpeg", "png", "gif"), extend -> {
        return MediaType.valueOf("image/" + (extend.equalsIgnoreCase("jpg") ? "jpeg" : extend));
    }),
    HLS(Arrays.asList("m3u8"), extend -> {
       return new MediaType("application", "vnd.apple.mpegurl");
    });


    private List<String> suffix;

    private Function<String, MediaType> mediaType;

    public static MediaType getMediaType(String filename) {
        String extend = StringUtils.substringAfterLast(filename, ".");
        return Arrays.stream(values()).filter(value -> value.suffix.contains(extend))
                .map(type -> type.getMediaType().apply(extend))
                .findFirst().orElse(MediaType.APPLICATION_OCTET_STREAM);
    }
}
