package com.flyfish.framework.file.service;


import com.flyfish.framework.file.domain.Attachment;
import com.flyfish.framework.file.utils.FileSizeUtils;
import com.flyfish.framework.service.impl.BaseServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.codec.multipart.FilePart;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;

@Service
public class AttachmentService extends BaseServiceImpl<Attachment> {

    private static String URL = "/api/attachment/";
    @Resource
    private FileService fileService;

    /**
     * 上传媒体文件
     *
     * @param part 文件
     * @return 结果
     */
    public Mono<Attachment> upload(FilePart part) {
        return upload(part, null);
    }

    /**
     * 上传媒体文件
     *
     * @param part 文件
     * @return 结果
     */
    public Mono<Attachment> upload(FilePart part, String name) {
        return fileService.saveLocal(part)
                .map(path -> {
                    Attachment attachment = Attachment.builder()
                            .size(FileSizeUtils.size(part.headers().getContentLength()))
                            .path(path)
                            .url(URL + path)
                            .build();
                    attachment.setName(StringUtils.isNotBlank(name) ? name : part.filename());
                    return create(attachment);
                });
    }
}
