package com.flyfish.framework.config.audit;

import com.flyfish.framework.auditor.BeanAuditor;
import com.flyfish.framework.auditor.BeanPoster;
import com.flyfish.framework.config.constants.UserCacheKeys;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.utils.Assert;
import com.flyfish.framework.utils.ReactiveRedisOperations;
import com.flyfish.framework.utils.StrengthUtils;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

@Component
@RequiredArgsConstructor
public class UserBeanAuditor implements BeanAuditor<User>, BeanPoster<User> {

    private final PasswordEncoder passwordEncoder;
    private final ReactiveRedisOperations reactiveRedisOperations;

    /**
     * 对实体进行审查，并补全相关字段
     *
     * @param entity 原数据
     */
    @Override
    public void audit(User entity) {
        if (null == entity.getId() && StringUtils.isNotBlank(entity.getPassword())) {
            Assert.isTrue(StrengthUtils.isValid(entity.getPassword()), "密码强度不够，至少应该包含数字、大小写字母、符号组合");
            entity.setPassword(passwordEncoder.encode(entity.getPassword()));
        }
        if (null == entity.getType()) {
            entity.setType(UserType.ADMIN);
        }
        if (null == entity.getEnable()) {
            entity.setEnable(true);
        }
        if (null == entity.getApp()) {
            entity.setApp(false);
        }
        if (null == entity.getCode()) {
            entity.setCode(entity.getUsername());
        }
    }

    /**
     * 对入库的实体进行审查，并执行额外功能
     *
     * @param data 原数据
     */
    @Override
    public void post(User data) {
        // 更新缓存
        reactiveRedisOperations.del(UserCacheKeys.get(data.getUsername())).subscribe();
    }
}
